# =====================================================================
# nosqllib.awk: NoSQL AWK function library.
#
# Copyright (c) 2006 Carlo Strozzi
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; version 2 dated June, 1991.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#
# =====================================================================

# =====================================================================
# __header()
#
# Load table header an populate __COLNAME[] e __COLPOS[]
# =====================================================================

function __header(						i) {

  if (__O_HEADER) print				# print header.

  gsub(/\001/,"")				# Remove SOH markers.

  gsub(/ +/,"")					# trim blanks in names.

  # Load the column position array.
  while (++i <= NF) {

    # Make sure we pick the first occurrence of duplicated column
    # names (it can happen after a join).

    if (__COLPOS[$i] == NULL) { __COLPOS[$i] = i; __COLNAME[i] = $i; }
  }
}

# =====================================================================
# int __appendmux(string name, anytype value,\
#				string muxfile [,boolean key])
#
# Append a name/value pair to a NoSQL mux file. If 'key' is set to '1'
# then the associate name/value pair is treated as a table key,
# otherwise it is a normal data field. Key names will have to be
# referenced as such also in the 'muxtotable' command that will be used
# to read the resulting file, through the '-K' option.
# The 'muxfile' argument must be in full-path; multiple mux files can
# be open at the same time.
# =====================================================================

function __appendmux(name,value,muxfile,key,			i) {

  # reject invalid filename.
  if (muxfile !~ /^\/\.*[a-zA-Z0-9]/) return _EFAULT

  # Special field name "." flushes the output file.
  if (name == ".") {
     for (i=1; i<=__MUXKEY[muxfile,0]; i++)
			print __MUXKEY[muxfile,i] > muxfile
     for (i=1; i<=__MUXDATA[muxfile,0]; i++)
			print __MUXDATA[muxfile,i] > muxfile
     close(muxfile)
     __MUXKEY[muxfile,0] = __MUXKEY[muxfile,0] = 0
     return 0
  }

  # reject invalid field names.
  if (name !~ /^[A-Za-z][A-Za-z0-9_]*$/) return _EINVAL

  # escape newlines in value.
  gsub(/\n/,"\n@",value)

  if (key) __MUXKEY[muxfile,++__MUXKEY[muxfile,0]] = name "\t" value
  else __MUXDATA[muxfile,++__MUXDATA[muxfile,0]] = name "\t" value

  return 0
}

# =====================================================================
# End of nosqllib.awk
# =====================================================================
